package ch.zhaw.projekt2.turnierverwaltung;


import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

import java.io.*;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Logger;


public class FileIO {
    private File mainDir;
    private File saves;

    private static final Logger logger = Logger.getLogger(FileIO.class.getName());


    public FileIO(String saveLocation) {
        this.mainDir = new File(saveLocation);
        if (!mainDir.exists()) {
            logger.fine("Creating main directory in given path" + saveLocation);
            mainDir.mkdir();
        } else {
            logger.finer("main directory already exists");
        }

        saves = new File(mainDir, "saves");
        if (!saves.exists()) {
            saves.mkdir();
            logger.fine("Creating save directory");
        } else {
            logger.finer("save directory already exists");
        }
    }

    public ObservableList<TournamentFile> getList() {
        logger.fine("Creating a List out of all Files in the save directory and returning it");
        ObservableList<TournamentFile> tournamentFiles = FXCollections.observableArrayList();
        for(File tournament : saves.listFiles()){
            tournamentFiles.add(new TournamentFile(tournament.toURI()));
        }
        return tournamentFiles;
    }

    public boolean tournamentExists(String name){
        for(TournamentFile file : getList()) {
            if(file.toString().toLowerCase().equals(name.toLowerCase())){
                return true;
            }
        }
        return false;
    }

    /**
     * @param tournamentFile
     * @return
     * @throws ClassNotFoundException
     * @throws IOException            File not found or not readable.
     */
    public Tournament loadTournament(File tournamentFile) throws IOException, ClassNotFoundException {
        if (tournamentFile == null) {
            logger.warning("Given tournament file is empty");
            throw new IllegalArgumentException("Tournament File is null");
        }
        Tournament tournament;
        logger.finer("Starting up Input Stream to read File");
        ObjectInputStream in = null;
        try {
            logger.fine("Setting up input file and reading it");
            FileInputStream fileInputStream = new FileInputStream(tournamentFile);
            in = new ObjectInputStream(fileInputStream);
            logger.finer("Starting to read tournament File");
            tournament = (Tournament) in.readObject();
        } catch (FileNotFoundException e) {
            logger.severe("Could not find tournament File");
            throw e;
        } catch (IOException e) {
            logger.severe("Failed to read File" + tournamentFile.getName());
            throw new IOException("Error while reading File",e);
        } catch (ClassNotFoundException e) {
            logger.severe("No definition for the class with the specified name could be found");
            throw new ClassNotFoundException("No definition for the class with the specified name could be found",e);
        } finally {
            if (in != null) {
                try {
                    logger.finer("Trying to close input stream");
                    in.close();
                } catch (IOException e) {
                    logger.severe("Failed to close input stream");
                    throw new IOException("Error while closing input stream",e);
                }
            }
        }
        return tournament;
    }

    public void saveTournament(Tournament tournament) throws IOException {
        if (tournament == null) {
            logger.warning("Given tournament file is empty");
            throw new IllegalArgumentException("Null tournament received");
        }
        File newSave = new File(saves, tournament.getName() + ".txt");
        ObjectOutputStream out = null;

        try {
            boolean newFile = newSave.createNewFile();
            out = new ObjectOutputStream(new FileOutputStream(newSave));
            out.writeObject(tournament);
            System.out.println("Save File" + tournament.getName() + ".txt being saved to " + saves.getAbsolutePath());

        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            }

        }

    }

    public class TournamentFile extends File{

        public TournamentFile(URI uri) {
            super(uri);
        }

        public String toString(){
            return getName().split("\\.")[0];
        }

    }

    public void deleteTournament(File tournamentFile) throws IOException {
        throw new UnsupportedOperationException("Method deleteTournament not implemented yet.");
    }

}
